/*
 * $Id: TicTacToe.java,v 1.2 1998/04/06 10:00:42 vmallet Exp $
 *
 *
 * $Log: TicTacToe.java,v $
 * Revision 1.2  1998/04/06 10:00:42  vmallet
 * Fixed some bugs, added some new ones.
 *
 * Revision 1.1  1998/03/11 02:23:52  vmallet
 * Initial revision
 *
 *
 */

package alt.tictac;

import game.*;

/**
 * <b>TicTacToe for Jags</b>
 * 
 * @version $Revision: 1.2 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public class TicTacToe extends Game implements TicTacToeConstants {
  
  static boolean won[] = new boolean[1 << 9];


  boolean whiteNext = true;

  int white;
  int black;

  int whiteId = -1;
  int blackId = -1;
  
  
  Player whitePlayer;
  Player blackPlayer;
  
  
  /**
   * Mark all positions with these bits set as winning.
   */
  static void isWon(int pos) {
    for (int i = 0 ; i < DONE ; i++) {
      if ((i & pos) == pos) {
	won[i] = true;
      }
    }
  }
  
  /**
   * Initialize all winning positions.
   */
  static {
    isWon((1 << 0) | (1 << 1) | (1 << 2));
    isWon((1 << 3) | (1 << 4) | (1 << 5));
    isWon((1 << 6) | (1 << 7) | (1 << 8));
    isWon((1 << 0) | (1 << 3) | (1 << 6));
    isWon((1 << 1) | (1 << 4) | (1 << 7));
    isWon((1 << 2) | (1 << 5) | (1 << 8));
    isWon((1 << 0) | (1 << 4) | (1 << 8));
    isWon((1 << 2) | (1 << 4) | (1 << 6));
  }


  /**
   * Creates a new TicTacToe
   */
  public TicTacToe() {
    super();
  }
  

  public Player createPlayer(String name, GameLinkFilter link) {
    return new TicServerPlayer(name, link);
  }
  

  public void addPlayer(Player player)
    throws TooManyPlayersException, ClosedGameException {
    
    super.addPlayer(player);
    
    if (whiteId == -1) {
      whiteId = player.getId();
      whitePlayer = player;
    } else {
      blackId = player.getId();
      blackPlayer = player;
    }
  }

  public void setupPlayer(Player player) {
    super.setupPlayer(player);
    //@@ yeah, VIOLENCE !
    player.getGameLink().sendBlindly(getStatusFrame());
  }



  public void removePlayer(Player player) {
    super.removePlayer(player);
    
    if (whiteId == player.getId()) {
      whiteId = -1;
      whitePlayer = null;
    } else {
      blackId = -1;
      blackPlayer = null;
    }

  }
    
  
  
  /**
   * User move.
   * @return true if legal
   */
  boolean yourMoveBlack(int m) {
    if (whiteNext)
      return false;
    
    if ((m < 0) || (m > 8)) {
      return false;
    }
    if (((black | white) & (1 << m)) != 0) {
      return false;
    }
    black |= 1 << m;
    whiteNext = true;
    return true;
  }

  boolean yourMoveWhite(int m) {
    if (!whiteNext)
      return false;
    if ((m < 0) || (m > 8)) {
      return false;
    }
    if (((black | white) & (1 << m)) != 0) {
      return false;
    }
    white |= 1 << m;
    whiteNext = false;
    return true;
  }
  


  /** q'n'sd !  */  
  boolean scored = false;
  
  
    /**
     * Figure what the status of the game is.
     */
    int status() {
      if (won[white]) {
	if (!scored) {
	  scored = true;
	  System.out.println("fouffa");
	  whitePlayer.setScore(whitePlayer.getScore() + 1);
	}
	return WIN;
      }
      if (won[black]) {
	if (!scored) {
	  scored = true;
	  System.out.println("fouffa 8");
	  blackPlayer.setScore(blackPlayer.getScore() + 1);
	}
	return LOSE;
      }
      if ((black | white) == DONE) {
	return STALEMATE;
      }
      return OK;
    }
  

    /**
     * Who goes first in the next game?
     */
    boolean first = true;



  public LinkFrame getStatusFrame() {
    LinkFrame frame = new LinkFrame();
    frame.setGroup(GAME_GROUP);
    frame.setRequest(RQ_SET_NEW_STATUS, status());
    Integer[] data = new Integer[2];
    data[0] = new Integer(white);
    data[1] = new Integer(black);
    frame.setData(data);

    return frame;
  }

  public void sendStatus() {
    notifyObservers(getStatusFrame());
  }


  public void frameReceived(Object link, LinkFrame frame) {
    int request = frame.getRequest();

    switch (request) {
    case RQ_PLAY_TURN:
      TicServerPlayer player = (TicServerPlayer) linkPlayers.get(link);
      if (player == null) {
	if (debug)
	  System.out.println("<TicTacToe> RQ_PLAY_TURN: unmatched player !");
	return;
      }
      int pId = player.getId();
      if (pId == frame.getSubRequest()) {
	System.out.println("<TicTacToe> wid=" + whiteId + "  bid=" + blackId + "   id=" + pId);
	int turn = ((Integer) frame.getData()).intValue();
	boolean res = (pId == whiteId) ? yourMoveWhite(turn) : yourMoveBlack(turn);

	if (res) {
	  sendStatus();
	} else System.out.println("<TicTacToe> RQ_PLAY_TURN: dontplayhere dork!");
	  
      } else if (debug) {
	System.out.println("<TicTacToe> RQ_PLAY_TURN: unmatched id !");
	System.out.println("<TicTacToe> wid=" + whiteId + "  bid=" + blackId + "   id=" + pId);
      }
      
      break;

    default:
      super.frameReceived(link, frame);
      break;
    }
  }

  
  public void start() {
  }
  

  public void repercution() {
  }

  public void yourTurn() {
  }


  public void run() {
    sendStatus();
  }
}
