/*
 * $Id: TicTacToeClient.java,v 1.3 1998/04/07 12:37:04 vmallet Exp $
 *
 *
 * $Log: TicTacToeClient.java,v $
 * Revision 1.3  1998/04/07 12:37:04  vmallet
 * Removed a debug output.
 *
 * Revision 1.2  1998/04/06 10:01:15  vmallet
 * Fixed some bugs, added some new ones.
 * Should handle first painting correctly.
 *
 * Revision 1.1  1998/03/11 02:25:37  vmallet
 * Initial revision
 *
 *
 */

package alt.tictac;

import game.*;

import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;

//import java.awt.*;
//import java.awt.event.*;
//import java.awt.image.*;
//import java.net.*;
//import java.applet.*;



/**
 * Client part of the <b>TicTacToe for Jags</b>
 *  
 * @version $Revision: 1.3 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public class TicTacToeClient extends ClientGame implements TicTacToeConstants {

  /**
   * Creates a new TicTacToeClient
   */
  public TicTacToeClient() {
  } 
  
  /**
   * 
   */
  public void make(GameLinkFilter link, int myPlayerId) {
    super.make(link, myPlayerId);

    gamePanel  = createWindow(130, 170);
    gamePanel2 = new MyPanel();
    gamePanel2.setSize(400,400);
    gamePanel.add(gamePanel2);
    gamePanel.validate();
//     gamePanel.repaint();
//     gamePanel2.repaint();
//     gamePanel2.repaint();
  }

  JLabel gamePanel2;
  

  /**
   * Creates a new particular <code>Player</code> for this game. This method 
   * should be overriden by children so the created player is of the right 
   * type.
   *
   * @param name the name of the new player to create
   * @param id   the id of the player, which should have been sent by the server.
   * @return the newly created player of the right type to be used in this
   *         game.
   */
  public Player createPlayer(int id, Object frameData) {
    return new TicClientPlayer(id, frameData);
  }
  

  
  /**
   * User move.
   * @return true if legal
   */
  boolean yourMove(int m) {
    if ((m < 0) || (m > 8)) {
      return false;
    }
    if (((black | white) & (1 << m)) != 0) {
      return false;
    }
    black |= 1 << m;
    return true;
  }
  
  
  
  // == implements GameLinkListener ============================
  
  public void frameReceived(Object link, LinkFrame frame) {
    int request = frame.getRequest();

    switch (request) {
    case RQ_SET_NEW_STATUS:
      status = frame.getSubRequest();
      Integer[] data = (Integer[]) frame.getData();
      white = data[0].intValue();
      black = data[1].intValue();
      gamePanel.repaint();
      gamePanel2.repaint();
      break;
      
    default:
      super.frameReceived(link, frame);
      break;
    }
  }
  
  /**
   * The game panel
   */
  Container gamePanel;
  
  int white;
  int black;
  
  
  /**
   * The image for white.
   */
  Image notImage = new ImageIcon("images/not.gif").getImage();
  
  /**
   * The image for black.
   */
  Image crossImage = new ImageIcon("images/cross.gif").getImage();
  
  
  /**
   * Figure what the status of the game is.
   */
  int status() {
    return status;
  }

  int status;


  public GameLinkFilter getGameLink() {
    return gameLink;
  }
  
  class MyPanel extends JLabel implements MouseListener {

    /**
     * Creates a new MyPanel
     */
    public MyPanel() {
      //      super(false);
      addMouseListener(this);
    }
    
    
    /**
     * Paint it.
     */
    public void paint(Graphics g) {
      Dimension d = getSize();
      g.setColor(Color.black);
      int xoff = d.width / 3;
      int yoff = d.height / 3;
      g.drawLine(xoff, 0, xoff, d.height);
      g.drawLine(2*xoff, 0, 2*xoff, d.height);
      g.drawLine(0, yoff, d.width, yoff);
      g.drawLine(0, 2*yoff, d.width, 2*yoff);

      int i = 0;
      for (int r = 0 ; r < 3 ; r++) {
	for (int c = 0 ; c < 3 ; c++, i++) {
	  if ((white & (1 << i)) != 0) {
	    g.drawImage(notImage, c*xoff + 1, r*yoff + 1, this);
	  } else if ((black & (1 << i)) != 0) {
	    g.drawImage(crossImage, c*xoff + 1, r*yoff + 1, this);
	  }
	}
      }
    }

    /**
     * The user has clicked in the applet. Figure out where
     * and see if a legal move is possible. If it is a legal
     * move, respond with a legal move (if possible).
     */
    public void mouseReleased(MouseEvent e) {
	int x = e.getX();
	int y = e.getY();

	switch (status()) {
	  case WIN:
	  case LOSE:
	  case STALEMATE:
	    System.out.println("<TicTacToeClient> The Game Has Ended !");
	    //	    play(getCodeBase(), "audio/return.au");
	    //	    white = black = 0;
	    //	    if (first) {
	    //		white |= 1 << (int)(Math.random() * 9);
	    //	    }
	    //	    first = !first;
	    //	    repaint();
	    return;
	}

	// Figure out the row/column
	Dimension d = getSize();
	int c = (x * 3) / d.width;
	int r = (y * 3) / d.height;
	
	LinkFrame frame = new LinkFrame();
	frame.setGroup(GAME_GROUP);
	frame.setRequest(RQ_PLAY_TURN, getMyPlayerId());
	frame.setData(new Integer(c + r * 3));
	getGameLink().sendBlindly(frame);
	
	if (false && yourMove(c + r * 3)) {
	    repaint();

	    switch (status()) {
	      case WIN:
		//		play(getCodeBase(), "audio/yahoo1.au");
		break;
	      case LOSE:
		//		play(getCodeBase(), "audio/yahoo2.au");
		break;
	      case STALEMATE:
		break;
	      default:
// 		if (myMove()) {
// 		    repaint();
// 		    switch (status()) {
// 		      case WIN:
// 			//			play(getCodeBase(), "audio/yahoo1.au");
// 			break;
// 		      case LOSE:
// 			//			play(getCodeBase(), "audio/yahoo2.au");
// 			break;
// 		      case STALEMATE:
// 			break;
// 		      default:
// 			//			play(getCodeBase(), "audio/ding.au");
// 		    }
// 		} else {
// 		  //		    play(getCodeBase(), "audio/beep.au");
// 		}
	    }
	} else {
	  //	    play(getCodeBase(), "audio/beep.au");
	}
    }


    public void mousePressed(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }


  } // class MyPanel

} // class TicTacToeClient


